using System;
using System.Configuration;
using System.Collections.Generic;
using System.Text;
using System.Xml;
using Guadagno.Utilities;

using Schedule;

namespace TaskPluginInterface
{

    /// <summary>
    /// Helper functions for reading the Schedule from the Configuration file and
    /// parsing it into its correct IScheduleItem
    /// </summary>
    public static class ScheduleConfig
    {

        #region Load
        /// <summary>
        /// Loads the schedule configuration from the passed XML configuration section
        /// </summary>
        /// <param name="configSection">A configuration section to parse</param>
        /// <returns>A List&lt;IScheduledItem&gt; items</returns>
        /// <exception cref="ConfigurationErrorsException" />
        public static List<IScheduledItem> Load(string configSection)
        {

            List<IScheduledItem> listSchedule = new List<IScheduledItem>();

            if (string.IsNullOrEmpty(configSection)) return listSchedule;

            try
            {
                XmlDocument xml = new XmlDocument();
                xml.LoadXml(configSection);

                XmlNode schedule = xml.SelectSingleNode("Schedule");

                foreach (XmlNode node in schedule.ChildNodes)
                {
                    switch (node.Name)
                    {
                        case "ScheduledTime":
                            listSchedule.Add(GetScheduledTime(node));
                            break;

                        case "SimpleInterval":
                            listSchedule.Add(GetSimpleInterval(node));
                            break;

                        case "SingleEvent":
                            listSchedule.Add(GetSingleEvent(node));
                            break;

                        case "BlockWrapper":
                            listSchedule.Add(GetBlockWrapper(node));
                            break;

                        case "EventQueue":
                            listSchedule.Add(GetEventQueue(node));
                            break;

                    }
                }

            }
            catch (Exception ex)
            {
                throw new ConfigurationErrorsException("There was an exception raised while loading the Schedule configuration.", ex);
            }


            return listSchedule;
        }
        #endregion Load

        #region IScheduleTime Types

        #region GetScheduledTime
        /// <summary>
        /// Reads the XmlNode and returns a ScheduledTime
        /// </summary>
        /// <param name="node">The ScheduledTime XML Node</param>
        /// <returns>A ScheduledTime</returns>
        /// <remarks>
        /// There is one valid attribute combination available for a ScheduledTime object.<br></br>
        /// The Node must have a <b>Base</b> and <b>Offset</b> attribute.<br></br>
        /// <example>&lt;ScheduledTime Base="" Offset="" /&gt;</example>
        /// </remarks>
        public static ScheduledTime GetScheduledTime(XmlNode node)
        {

            // Options Available
            // Base, Offset

            string sBase;
            string sOffset;

            sBase = node.Attributes["Base"].Value;
            sOffset = node.Attributes["Offset"].Value;

            if (sBase == null) throw new ConfigurationErrorsException("Missing the Base attribute for ScheduledTime", node);
            if (sOffset == null) throw new ConfigurationErrorsException("Missing the Offset attribute for ScheduledTime", node);


            return new ScheduledTime(sBase, sOffset);
        }
        #endregion GetScheduledTime

        #region GetSimpleInterval

        /// <summary>
        /// Reads the XmlNode and returns a SimpleInterval
        /// </summary>
        /// <param name="node">The SimpleInterval XmlNode to parse.</param>
        /// <returns>A SimpleInterval</returns>
        /// <remarks>
        /// There are three available combinations for the SimpleInterval node.<br></br>
        /// StartTime, Interval, EndTime<br></br>
        /// StartTime, Interval<br></br>
        /// StartTime, Interval, Count<br></br>
        /// </remarks>
        /// <example>
        /// &lt;SimpleInterval <b>StartTime</b>="" <b>Interval</b>="" <b>EndTime</b>="" /&gt;<br></br> 
        /// &lt;SimpleInterval <b>StartTime</b>="" <b>Interval</b>="" /&gt;<br></br> 
        /// &lt;SimpleInterval <b>StartTime</b>="" <b>Interval</b>="" <b>Count</b>="" /&gt;<br></br> 
        /// </example>
        public static SimpleInterval GetSimpleInterval(XmlNode node)
        {

            // Options Available for SimpleInterval
            // StartTime, Interval, EndTime
            // StartTime, Interval
            // StartTime, Interval, Count

            DateTime? startTime = null;
            DateTime? endTime = null;
            TimeSpan? interval = null;
            int? count = null;

            if (node.Attributes["StartTime"] != null) { TryParse.TryNullableDateTime(node.Attributes["StartTime"].Value, out startTime); }

            if (node.Attributes["Interval"] != null) { TryParse.TryNullableTimeSpan(node.Attributes["Interval"].Value, out interval); }

            if (node.Attributes["EndTime"] != null) { TryParse.TryNullableDateTime(node.Attributes["EndTime"].Value, out endTime); }

            if (node.Attributes["Count"] != null) { TryParse.TryNullableInt(node.Attributes["Count"].Value, out count); }

            // Determine which Constructor to call
            // StartTime and Interval are required

            if (startTime == null) throw new ConfigurationErrorsException("Missing the StartTime attribute for SimpleInterval", node);
            if (interval == null) throw new ConfigurationErrorsException("Missing the Interval attribute for SimpleInterval", node);

            if (endTime != null)
            {
                return new SimpleInterval(startTime.Value, interval.Value, endTime.Value);
            }
            if (count != null)
            {
                return new SimpleInterval(startTime.Value, interval.Value, count.Value);
            }

            // The default
            return new SimpleInterval(startTime.Value, interval.Value);

        }

        #endregion GetSimpleInterval

        #region GetSingleEvent
        /// <summary>
        /// Read the XmlNode and returns a SingleEvent
        /// </summary>
        /// <param name="node">The SingleEvent XmlNode to parse.</param>
        /// <returns>A SingleEvent</returns>
        /// <remarks>There is one available option for the SingleEvent node.</remarks>
        /// <example>&lt;SingleEvent <b>EventTime</b>=""&gt;</example>
        public static SingleEvent GetSingleEvent(XmlNode node)
        {
            // Options Available
            // EventTime

            DateTime? eventTime = null;

            if (node.Attributes["EventTime"] != null) { TryParse.TryNullableDateTime(node.Attributes["EventTime"].Value, out eventTime); }
            if (eventTime != null)
            {
                return new SingleEvent(eventTime.Value);
            }
            else
            {
                throw new ConfigurationErrorsException("Missing the EventTime attribute for SingleEvent", node);
            }

        }

        #endregion GetSingleEvent

        #region GetBlockWrapper
        /// <summary>
        /// Reads the XmlNode and returns a SimpleInterval
        /// </summary>
        /// <param name="node">The BlockWrapper XmlNode</param>
        /// <returns>A BlockWrapper</returns>
        /// <remarks>
        /// The Blockwrapper requires a Base, BeginOffset and EndOffset attributes and a Child node of an IScheduledItem
        /// </remarks>
        /// <example>
        /// &lt;BlockWrapper <b>Base</b>="" <b>BeginOffset</b>="" <b>EndOffset</b>&gt;<br></br>
        ///   &lt;SimpleInterval <b>StartTime</b>="" <b>Interval</b>="" <b>EndTime</b>="" /&gt;<br></br> 
        /// &lt;BlockWrapper&gt;
        /// </example>
        public static BlockWrapper GetBlockWrapper(XmlNode node)
        {

            // Options Available
            // IScheduleItem, Base, BeginOffset, EndOffset

            string sBase, beginOffset, endOffset;
            IScheduledItem scheduledItem = null;

            sBase = node.Attributes["Base"].Value;
            beginOffset = node.Attributes["BeginOffset"].Value;
            endOffset = node.Attributes["EndOffset"].Value;

            if (sBase == null) throw new ConfigurationErrorsException("Missing the Base attribute for BlockWrapper", node);
            if (beginOffset == null) throw new ConfigurationErrorsException("Missing the BeginOffset attribute for BlockWrapper", node);
            if (endOffset == null) throw new ConfigurationErrorsException("Missing the EndOffset attribute for BlockWrapper", node);

            XmlNode scheduleNode = node.FirstChild;

            if (scheduleNode == null) throw new ConfigurationErrorsException("Missing an IScheduleItem for the BlockWrapper", node);

            switch (scheduleNode.Name)
            {
                case "ScheduledTime":
                    scheduledItem = GetScheduledTime(scheduleNode);
                    break;

                case "SimpleInterval":
                    scheduledItem = GetSimpleInterval(scheduleNode);
                    break;

                case "SingleEvent":
                    scheduledItem = GetSingleEvent(scheduleNode);
                    break;

            }

            return new BlockWrapper(scheduledItem, sBase, beginOffset, endOffset);
        }

        #endregion GetBlockWrapper

        #region GetEventQueue
        /// <summary>
        /// Reads the XmlNode and returns an EventQueue
        /// </summary>
        /// <param name="node">The EventQueue node to parse</param>
        /// <returns>An EventQueue</returns>
        /// <remarks>
        /// The EventQueue contains one or more IScheduleItems
        /// </remarks>
        /// <example>
        /// &lt;EventQueue&gt; <br></br>
        ///     &lt;SimpleInterval <b>StartTime</b>="" <b>Interval</b>="" <b>EndTime</b>="" /&gt;<br></br> 
        ///     &lt;SimpleInterval <b>StartTime</b>="" <b>Interval</b>="" /&gt;<br></br> 
        ///     &lt;SimpleInterval <b>StartTime</b>="" <b>Interval</b>="" <b>Count</b>="" /&gt;<br></br> 
        /// &lt;/EventQueue&gt;
        /// </example>
        public static EventQueue GetEventQueue(XmlNode node)
        {

            EventQueue queue = new EventQueue();

            if (node.ChildNodes.Count == 0) throw new ConfigurationErrorsException("There are no IScheduleItems for the EventQueue", node);

            foreach (XmlNode child in node.ChildNodes)
            {
                switch (child.Name)
                {
                    case "ScheduledTime":
                        queue.Add(GetScheduledTime(child));
                        break;

                    case "SimpleInterval":
                        queue.Add(GetSimpleInterval(child));
                        break;

                    case "SingleEvent":
                        queue.Add(GetSingleEvent(child));
                        break;

                    case "BlockWrapper":
                        queue.Add(GetBlockWrapper(child));
                        break;

                    case "EventQueue":
                        queue.Add(GetEventQueue(child));
                        break;
                }
            }

            return queue;
        }
        #endregion GetEventQueue

        #endregion IScheduleTime Types

        #region Helper Functions

        #endregion Helper Functions

    }
}
