using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

using System.IO;
using System.Reflection;
using TaskPluginInterface;

namespace TaskPluginTest
{
    public partial class frmMain : Form
    {
        public frmMain()
        {
            InitializeComponent();
        }

        #region Private Variables

        private ITaskPlugin m_ActivePlugin = null;
        private string m_Default_Plugin_Folder = "Plugins";
        private bool m_Cancel = false;
        private bool m_PluginExecuting = false;

        #endregion Private Variables

        #region Events

        #region Button Events

        private void btnBrowseFolder_Click(object sender, EventArgs e)
        {
            fldBrowser.SelectedPath = Path.Combine(Application.StartupPath, "Plugins");
            DialogResult result = fldBrowser.ShowDialog();

            if (result != DialogResult.Cancel)
            {
                txtFolder.Text = fldBrowser.SelectedPath;
                cboPlugin.DataSource = null;
                cboPluginFiles.DataSource = null;
                ClearPluginInfo();
            }
        }

        private void btnLoadPlugins_Click(object sender, EventArgs e)
        {
            // Reload the files that contain Plugins.
            if (Directory.Exists(txtFolder.Text))
            {
                ListPlugins(txtFolder.Text);
            }
        }

        private void btnExecute_Click(object sender, EventArgs e)
        {
            if (m_ActivePlugin != null)
            {
                try
                {
                    m_Cancel = false;

                    EnableForm(false);

                    taskProgress.Value = 0;
                    itemProgress.Value = 0;
                    lblMessage.Text = "";

                    m_PluginExecuting = true;
                    ExecuteResult result = m_ActivePlugin.Execute();
                    m_PluginExecuting = false;
                    MessageBox.Show(string.Format("The result of Execute was '{0}'", Enum.GetName(typeof(ExecuteResult), result)), "Executed Task!", MessageBoxButtons.OK, MessageBoxIcon.Information);
                }
                catch (Exception ex)
                {
                    MessageBox.Show(ex.ToString(), "Error executing the task!", MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
                finally
                {
                    EnableForm(true);
                }
            }
        }

        private void btnCancel_Click(object sender, EventArgs e)
        {
            m_Cancel = true;
        }
        #endregion Button Events

        #region ComboBox Events

        private void cboPluginTypes_SelectedIndexChanged(object sender, EventArgs e)
        {
            ListPlugins(txtFolder.Text);
        }

        private void cboPluginFiles_SelectedIndexChanged(object sender, EventArgs e)
        {
            // Need to get the list of Plugins from the file
            ClearPluginInfo();

            cboPlugin.DataSource = null;

            if (cboPluginFiles.SelectedItem != null)
            {
                cboPlugin.DataSource = Utilities.GetPlugins(((FileInfo)cboPluginFiles.SelectedItem).FullName);
                cboPlugin.DisplayMember = "Name";
            }
        }

        private void cboPlugin_SelectedIndexChanged(object sender, EventArgs e)
        {
            // Need to load the plug-in information
            ClearPluginInfo();

            if (cboPlugin.SelectedItem != null)
            {
                DisplayPluginInfo(cboPluginFiles.SelectedItem.ToString(), cboPlugin.SelectedItem.ToString());
            }

        }

        #endregion ComboBox Events

        #region Form Events

        private void frmMain_Load(object sender, EventArgs e)
        {
            fldBrowser.RootFolder = Environment.SpecialFolder.MyComputer;

            string pluginFolder = TaskPluginInterface.Utilities.GetPluginFolder(m_Default_Plugin_Folder);

            fldBrowser.SelectedPath = pluginFolder;
            txtFolder.Text = fldBrowser.SelectedPath;

            cboPluginTypes.Items.AddRange(Enum.GetNames(typeof(PluginType)));
            cboPluginTypes.Items.Add("All");
            cboPluginTypes.SelectedItem = "All";

            this.Icon = Properties.Resources.Icon_Clock;

        }

        void frmMain_Closing(object sender, System.ComponentModel.CancelEventArgs e)
        {
            if (m_PluginExecuting)
                m_Cancel = true;
        }

        #endregion Form Events

        #endregion Events

        #region ITaskPlugin Events

        void activePlugin_EventProcessing(object sender, PluginEventArgs e)
        {
            e.Cancel = m_Cancel;
            RefreshProgress(e);
        }

        void activePlugin_EventExceptionOccurred(object sender, PluginEventArgs e)
        {
            RefreshProgress(e);
            MessageBox.Show(e.RaisedException.ToString(), "Error executing the task!", MessageBoxButtons.OK, MessageBoxIcon.Error);
        }

        void activePlugin_EventStart(object sender, PluginEventArgs e)
        {
            RefreshProgress(e);
        }

        void activePlugin_EventEnd(object sender, PluginEventArgs e)
        {
            RefreshProgress(e);
        }

        void RefreshProgress(PluginEventArgs e)
        {
            if (e.Progress.TotalTasks == 0)
            {
                taskProgress.Value = 0;
            }
            else
            {
                taskProgress.Maximum = e.Progress.TotalTasks;
                taskProgress.Value = e.Progress.CurrentTask;
                string msg = string.Format("Working on task {0} of {1}.", taskProgress.Value, taskProgress.Maximum);
                tt.SetToolTip(taskProgress, msg);
            }

            if (e.Progress.TotalItems == 0)
            {
                itemProgress.Value = 0;
            }
            else
            {
                itemProgress.Maximum = e.Progress.TotalItems;
                itemProgress.Value = e.Progress.CurrentItem;
                string msg = string.Format("Working on item {0} of {1}.", itemProgress.Value, itemProgress.Maximum);
                tt.SetToolTip(itemProgress, msg);
            }

            if (string.IsNullOrEmpty(e.Message))
            {
                lblMessage.Text = "";
            }
            else
            {
                lblMessage.Text = e.Message;
            }

            itemProgress.Refresh();
            taskProgress.Refresh();
            Application.DoEvents();
        }
        #endregion ITaskPlugin Events

        #region Helper Methods

        /// <summary>
        /// Populates the combo box with a list of valid Plugins.
        /// </summary>
        /// <param name="folder">The folder in which you want to get a plug-in list</param>
        private void ListPlugins(string folder)
        {
            ClearPluginInfo();

            cboPluginFiles.DataSource = null;
            cboPlugin.DataSource = null;

            if (Directory.Exists(folder) == false)
            {
                return;
            }

            if (cboPluginTypes.SelectedItem != null)
            {
                string pluginTypeName = cboPluginTypes.SelectedItem.ToString();
                {
                    if (pluginTypeName == "All")
                    {
                        cboPluginFiles.DataSource = TaskPluginInterface.Utilities.GetPluginFileList(folder);
                    }
                    else
                    {
                        PluginType pluginType = (PluginType)Enum.Parse(typeof(PluginType), pluginTypeName);
                        cboPluginFiles.DataSource = TaskPluginInterface.Utilities.GetPluginFileList(folder, pluginType);
                    }
                }
            }
            cboPluginFiles.DisplayMember = "Name";

        }

        /// <summary>
        /// Enables or disables the controls on the form
        /// </summary>
        /// <param name="OnOrNot">True to enable the form. False to disable it.</param>
        private void EnableForm(bool OnOrNot)
        {

            if (OnOrNot == false)
            {
                this.Cursor = Cursors.WaitCursor;
            }

            txtFolder.Enabled = OnOrNot;
            btnBrowseFolder.Enabled = OnOrNot;
            btnLoadPlugins.Enabled = OnOrNot;
            btnExecute.Enabled = OnOrNot;
            cboPluginTypes.Enabled = OnOrNot;
            cboPluginFiles.Enabled = OnOrNot;
            cboPlugin.Enabled = OnOrNot;
            grdSchedule.Enabled = OnOrNot;

            btnCancel.Enabled = !OnOrNot;

            if (OnOrNot == true)
            {
                this.Cursor = Cursors.Default;
            }
        }

        /// <summary>
        /// Clears the plug-in information from the labels
        /// </summary>
        private void ClearPluginInfo()
        {
            lblName.Text = "";
            lblDescription.Text = "";
            lblExtension.Text = "";
            lblVersion.Text = "";
            lblAuthor.Text = "";
            lblExtensionDescription.Text = "";
            grdSchedule.DataSource = null;
            taskProgress.Value = 0;
            itemProgress.Value = 0;

            taskProgress.Refresh();
            itemProgress.Refresh();
        }

        #endregion Helper Methods

        #region ITaskPlugin Methods

        /// <summary>
        /// Displays the plug-in information.
        /// </summary>
        /// <param name="filename">The filename that contains the Plugin</param>
        /// <param name="className">The class name to load</param>
        private void DisplayPluginInfo(string filename, string className)
        {

            TaskPluginInterface.ITaskPlugin plugin = TaskPluginInterface.Utilities.GetPlugin(filename, className);

            lblMessage.Text = "";
            taskProgress.Value = 0;

            if (plugin != null)
            {
                m_ActivePlugin = plugin;
                lblName.Text = plugin.Name;
                lblDescription.Text = plugin.Description;
                lblExtension.Text = plugin.Extension;
                lblExtensionDescription.Text = plugin.ExtensionDescription;
                lblVersion.Text = plugin.Version;
                lblAuthor.Text = plugin.Author;

                if (plugin.Extension != "")
                {
                    ofd.Filter = string.Format("{0} files ({1})|{1}|All Files (*.*)|*.*", plugin.ExtensionDescription, plugin.Extension);
                    ofd.FilterIndex = 0;
                }
                else
                {
                    ofd.Filter = "All Files (*.*)|*.*";
                    ofd.FilterIndex = 0;
                }


                // /Load the Schedule
                DataSet ds = new DataSet();
                DataTable dt = ds.Tables.Add("Parameters");
                dt.Columns.Add(new DataColumn("Type", typeof(System.Type)));
                dt.Columns.Add(new DataColumn("Date", typeof(DateTime)));


                foreach (Schedule.IScheduledItem schedule in plugin.Schedule)
                {
                    DataRow dr = dt.NewRow();
                    dr["Type"] = schedule.GetType();
                    dr["Date"] = schedule.NextRunTime(DateTime.Now, false);
                    dt.Rows.Add(dr);
                }

                grdSchedule.DataSource = dt;

                // Attach the events
                m_ActivePlugin.EventEnd += new OnEnd(activePlugin_EventEnd);
                m_ActivePlugin.EventStart += new OnStart(activePlugin_EventStart);
                m_ActivePlugin.EventExceptionOccurred += new OnExceptionOccurred(activePlugin_EventExceptionOccurred);
                m_ActivePlugin.EventProcessing += new OnProcessing(activePlugin_EventProcessing);

            }

        }

        #endregion ITaskPlugin Methods


    }
}