using System;
using System.Collections.Generic;
using System.IO;
using System.Xml;
using TaskPluginInterface;
using Schedule;

namespace ApplicationPlugins
{
    /// <summary>
    /// A plugin to copy files from source to destination.
    /// </summary>
    [Plugin(PluginType.Executable)]
    public class FileCopy : ITaskPlugin
    {

        #region Private Variables

        // For Properties

        // For configurtion
        private FileCopyConfig _config;
        private FileCopyConfig.ConfigDirectory[] _dirs;

        // For Progress
        private Utilities.Progress _progress;

        // Other Variables
        private readonly log4net.ILog _log = log4net.LogManager.GetLogger(System.Reflection.MethodBase.GetCurrentMethod().DeclaringType);

        // Cancel button was clicked
        private bool _cancel;

        #region Private Constants


        #endregion Private Constants

        #region Private Enums

        #endregion Private Enums

        #endregion Private Variables

        #region Constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="FileCopy"/> class.
        /// </summary>
        /// <param name="progress">The progress.</param>
        public FileCopy(Utilities.Progress progress)
        {
            loadConfiguration();

            _progress = progress;
        }

        #endregion Constructors

        #region ITaskPlugin Members

        #region ITaskPlugin Properties

        /// <summary>
        /// The name of the plug-in.
        /// </summary>
        /// <value></value>
        public string Name
        {
            get { return Properties.FileCopyPlugin.PluginName; }
            set { throw new NotImplementedException(); }
        }

        /// <summary>
        /// The description of the plug-in.
        /// </summary>
        /// <value></value>
        public string Description
        {
            get { return Properties.FileCopyPlugin.PluginDescription; }
        }

        /// <summary>
        /// The extension that this plug-in handles.
        /// </summary>
        /// <value></value>
        public string Extension
        {
            get { return Properties.FileCopyPlugin.PluginExtension; }
        }

        /// <summary>
        /// The description of the extension that this plug-in handles.
        /// </summary>
        /// <value></value>
        public string ExtensionDescription
        {
            get { return Properties.FileCopyPlugin.PluginExtensionDescription; }
        }

        /// <summary>
        /// The version of the plug-in.
        /// </summary>
        /// <value></value>
        public string Version
        {
            get { return Properties.FileCopyPlugin.PluginVersion; }
        }

        /// <summary>
        /// The author of the plug-in.
        /// </summary>
        /// <value></value>
        public string Author
        {
            get { return Properties.FileCopyPlugin.PluginAuthor; }
        }

        /// <summary>
        /// Returns the schedule for the plug-in.
        /// </summary>
        /// <value></value>
        public List<IScheduledItem> Schedule { get; private set; }

        #endregion ITaskPlugin Properties

        #region ITaskPlugin Methods

        /// <summary>
        /// Executes the plug-in.
        /// </summary>
        /// <returns>A <see cref="ExecuteResult"/></returns>
        public ExecuteResult Execute()
        {

            #region Variables

            #endregion Variables

            #region Data Validation

            #endregion Data Validation

            try
            {
                // Setup the Progress Information
                _progress.TotalTasks = 1;
                _progress.TotalItems = 1;
                _progress.CurrentTask = 0;
                _progress.CurrentItem = 0;
                _cancel = false;

                _log.Debug(Properties.Resources.Execute_Started);

                // If the EventStart event has been subscribed to let's fire it.
                if (EventStart != null)
                    EventStart(this, new PluginEventArgs(_progress, "Starting the task.", null));

                DoWork();

                // If the EventStart event has been subscribed to let's fire it.
                if (EventEnd != null)
                {
                    if (_cancel)
                    {
                        EventEnd(this, new PluginEventArgs(_progress, "Task execution was canceled.", null));
                    }
                    else
                    {
                        EventEnd(this, new PluginEventArgs(_progress, "Task execution is complete.", null));
                    }
                }

                // Exit 
                if (_cancel)
                {
                    _log.DebugFormat(Properties.Resources.Execute_Ended, ExecuteResult.Canceled);
                    return ExecuteResult.Canceled;
                }
                _log.DebugFormat(Properties.Resources.Execute_Ended, ExecuteResult.Ok);
                return ExecuteResult.Ok;
            }
            catch (Exception ex)
            {
                if (EventExceptionOccurred != null)
                    EventExceptionOccurred(this, new PluginEventArgs(_progress, "Exception Raised", ex));

                // Exit with an Exception
                return ExecuteResult.Exception;
            }

        }

        /// <summary>
        /// Cancels the execution of a plug-in
        /// </summary>
        /// <returns>
        /// True, if the cancel request was excepted, otherwise false.
        /// </returns>
        public bool Cancel()
        {
            _log.Debug(Properties.Resources.CancelMethodExecuted);
            _cancel = true;
            return true;
        }
        #endregion ITaskPlugin Methods

        #region ITaskPlugin Events
        /// <summary>
        /// This is used to notify the plug-in host about the starting of the Execute Method.
        /// </summary>
        public event OnStart EventStart;
        /// <summary>
        /// This is used to notify plug-in host about the ending of the Execute Method.
        /// </summary>
        public event OnEnd EventEnd;
        /// <summary>
        /// This is used to notify plug-in host about the processing of the Execute Method.
        /// </summary>
        public event OnProcessing EventProcessing;
        /// <summary>
        /// This is used to notify plug-in host about any Exceptions that have occurred.
        /// </summary>
        public event OnExceptionOccurred EventExceptionOccurred;
        #endregion ITaskPlugin Events

        #endregion

        #region Private Methods

        /// <summary>
        /// This method reads the "Directories" from the configuration file and copies
        /// the files.
        /// </summary>
        /// <returns>An <see cref="TaskPluginInterface.ExecuteResult"/> value, with the results of
        /// the method.</returns>
        private ExecuteResult DoWork()
        {

            // Reset the currentItem counter
            _progress.CurrentItem = 0;
            _progress.CurrentTask = 0;

            // Set the numbers of items to process
            _progress.TotalTasks = _dirs.Length;

            _log.DebugFormat(Properties.Resources.FoundDirectories, _dirs.Length);

            foreach (FileCopyConfig.ConfigDirectory configDir in _dirs)
            {
                _progress.CurrentItem = 0;

                // Get the file list that matches the FileMask
                string[] files = Directory.GetFiles(configDir.FromDir, configDir.FileMask);
                // Update the Total Items count
                _progress.TotalItems = files.Length;

                _log.DebugFormat(Properties.Resources.FoundFiles, files.Length, configDir.FromDir, configDir.FileMask);

                // See if we need to create the new directory
                if (Directory.Exists(configDir.ToDir) == false)
                {
                    _log.DebugFormat(Properties.Resources.CreatingDirectory, configDir.ToDir);
                    Directory.CreateDirectory(configDir.ToDir);
                }

                // Loop through each file.
                foreach (string file in files)
                {
                    string msg = string.Format(Properties.Resources.CopyingFile, file, configDir.FromDir, configDir.ToDir);

                    // Create the new filename
                    FileInfo copyFile = new FileInfo(file);

                    // get the name of the new file.
                    string newFile = Path.Combine(configDir.ToDir, copyFile.Name);
                    copyFile.CopyTo(newFile, true);

                    _progress.CurrentItem++;
                    if (EventProcessing != null)
                    {
                        PluginEventArgs evt = new PluginEventArgs(_progress, msg, null);
                        EventProcessing(this, evt);
                        if (evt.Cancel)
                        {
                            _log.Debug(Properties.Resources.CancelRequestRecieved);
                            _cancel = true;
                            break;
                        }
                    }
                }

                if (_cancel)
                    break;

                if (EventProcessing != null)
                {
                    PluginEventArgs evt = new PluginEventArgs(_progress, "Working", null);
                    EventProcessing(this, evt);
                    if (evt.Cancel)
                    {
                        _log.Debug(Properties.Resources.CancelRequestRecieved);
                        _cancel = true;
                        break;
                    }
                }

                _progress.CurrentTask++;
            }

            if (_cancel)
                return ExecuteResult.Canceled;
            
            return ExecuteResult.Ok;
        }

        #endregion Private Methods

        #region Helper Functions

        #region loadConfiguration
        private void loadConfiguration()
        {

            string configFile = System.Reflection.Assembly.GetExecutingAssembly().Location + ".Settings.xml";
            _config = new FileCopyConfig(null, configFile);

            if (!string.IsNullOrEmpty(_config.Log4NetConfig))
            {
                XmlDocument xml = new XmlDocument();
                xml.LoadXml(_config.Log4NetConfig);
                log4net.Config.XmlConfigurator.Configure(xml.DocumentElement);
            }

            this.Schedule = ScheduleConfig.Load(_config.Schedule);

            _dirs = _config.Directories;

            _log.Debug(Properties.Resources.ConfigurationLoaded);
        }
        #endregion loadConfiguration

        #endregion Helper Functions
    }
}
