using System;
using System.Collections.Generic;
using System.Xml;
using TaskPluginInterface;
using Schedule;

namespace ApplicationPlugins
{
    /// <summary>
    /// A class for providing the basics for creating a plugin.
    /// </summary>
    [Plugin(PluginType.Unknown)]
    public class PluginShell : ITaskPlugin
    {

        #region Private Variables

        // For configuration
        private PluginShellConfig _config;

        // For Progress
        private Utilities.Progress _progress;

        // Other Variables
        private readonly log4net.ILog _log = log4net.LogManager.GetLogger(System.Reflection.MethodBase.GetCurrentMethod().DeclaringType);

        // Cancel button was clicked
        private bool _cancel;

        #region Private Enums

        #endregion Private Enums

        #region Private Constants

        #endregion Private Constants

        #endregion Private Variables

        #region Constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="PluginShell"/> class.
        /// </summary>
        public PluginShell(Utilities.Progress progress)
        {
            loadConfiguration();

            _progress = progress;
        }

        #endregion Constructors

        #region ITaskPlugin Members

        #region ITaskPlugin Properties

        /// <summary>
        /// The name of the plug-in.
        /// </summary>
        /// <value></value>
        public string Name
        {
            get { return Properties.PluginShellPlugin.PluginName; }
        }

        /// <summary>
        /// The description of the plug-in.
        /// </summary>
        /// <value></value>
        public string Description
        {
            get { return Properties.PluginShellPlugin.PluginDescription; }
        }

        /// <summary>
        /// The extension that this plug-in handles.
        /// </summary>
        /// <value></value>
        public string Extension
        {
            get { return Properties.PluginShellPlugin.PluginExtension; }
        }

        /// <summary>
        /// The description of the extension that this plug-in handles.
        /// </summary>
        /// <value></value>
        public string ExtensionDescription
        {
            get { return Properties.PluginShellPlugin.PluginExtensionDescription; }
        }

        /// <summary>
        /// The version of the plug-in.
        /// </summary>
        /// <value></value>
        public string Version
        {
            get { return Properties.PluginShellPlugin.PluginVersion; }
        }

        /// <summary>
        /// The author of the plug-in.
        /// </summary>
        /// <value></value>
        public string Author
        {
            get { return Properties.PluginShellPlugin.PluginAuthor; }
        }

        /// <summary>
        /// Returns the schedule for the plug-in.
        /// </summary>
        /// <value></value>
        public List<IScheduledItem> Schedule
        {
            get; set;
        }

        #endregion ITaskPlugin Properties

        #region ITaskPlugin Methods

        /// <summary>
        /// Executes the plug-in.
        /// </summary>
        /// <returns>A <see cref="ExecuteResult"/></returns>
        public ExecuteResult Execute()
        {

            #region Variables

            #endregion Variables

            #region Data Validation

            #endregion Data Validation

            try
            {
                // Setup the Progress Information
                _progress.TotalTasks = 1;
                _progress.TotalItems = 1;
                _progress.CurrentTask = 0;
                _progress.CurrentItem = 0;
                _cancel = false;

                _log.Debug(Properties.Resources.Execute_Started);

                // If the EventStart event has been subscribed to let's fire it.
                if (EventStart != null)
                    EventStart(this, new PluginEventArgs(_progress, "Starting the task.", null));

                doWork();

                // If the EventStart event has been subscribed to let's fire it.
                if (EventEnd != null)
                {
                    if (_cancel)
                    {
                        EventEnd(this, new PluginEventArgs(_progress, "Task execution was canceled.", null));
                    }
                    else
                    {
                        EventEnd(this, new PluginEventArgs(_progress, "Task execution is complete.", null));
                    }
                }

                // Exit 
                if (_cancel)
                {
                    _log.DebugFormat(Properties.Resources.Execute_Ended, ExecuteResult.Canceled);
                    return ExecuteResult.Canceled;
                }
                _log.DebugFormat(Properties.Resources.Execute_Ended, ExecuteResult.Ok);
                return ExecuteResult.Ok;
            }
            catch (Exception ex)
            {
                if (EventExceptionOccurred != null)
                    EventExceptionOccurred(this, new PluginEventArgs(_progress, "", ex));

                // Exit with an Exception
                return ExecuteResult.Exception;
            }

        }

        /// <summary>
        /// Cancels the execution of a plug-in
        /// </summary>
        /// <returns>
        /// True, if the cancel request was excepted, otherwise false.
        /// </returns>
        public bool Cancel()
        {
            _log.Debug(Properties.Resources.CancelMethodExecuted);
            _cancel = true;
            return true;
        }
        #endregion ITaskPlugin Methods

        #region ITaskPlugin Events
        /// <summary>
        /// This is used to notify the plug-in host about the starting of the Execute Method.
        /// </summary>
        public event OnStart EventStart;
        /// <summary>
        /// This is used to notify plug-in host about the ending of the Execute Method.
        /// </summary>
        public event OnEnd EventEnd;
        /// <summary>
        /// This is used to notify plug-in host about the processing of the Execute Method.
        /// </summary>
        public event OnProcessing EventProcessing;
        /// <summary>
        /// This is used to notify plug-in host about any Exceptions that have occurred.
        /// </summary>
        public event OnExceptionOccurred EventExceptionOccurred;
        #endregion ITaskPlugin Events

        #endregion

        #region Private Methods

        /// <summary>
        /// Sample subroutine to demonstrate how to execute work.
        /// </summary>
        /// <returns></returns>
        private ExecuteResult doWork()
        {

            const int longLoop = 120;
            // Reset the currentItem counter
            _progress.CurrentItem = 0;
            // Set the numbers of items to process
            _progress.TotalItems = longLoop;

            _log.Debug(Properties.Resources.StartingToDoWork);

            for (int x = 1; x < longLoop; x++)
            {
                // Just Sleep for a half of a second
                System.Threading.Thread.Sleep(50);

                _log.DebugFormat(Properties.Resources.WorkingOn, x);

                // Update the current item
                _progress.CurrentItem = x;
                if (EventProcessing != null)
                {
                    PluginEventArgs evt = new PluginEventArgs(_progress, "Working", null);
                    EventProcessing(this, evt);
                    if (evt.Cancel)
                    {
                        _log.Debug(Properties.Resources.CancelRequestRecieved);
                        _cancel = true;
                        break;
                    }
                }
            }

            if (_cancel)
                return ExecuteResult.Canceled;
            _progress.CurrentTask++;
            return ExecuteResult.Ok;
        }

        #endregion Private Methods

        #region Helper Functions

        #region loadConfiguration

        private void loadConfiguration()
        {

            string configFile = System.Reflection.Assembly.GetExecutingAssembly().Location + ".Settings.xml";
            _config = new PluginShellConfig(null, configFile);

            if (!string.IsNullOrEmpty(_config.Log4NetConfig))
            {
                XmlDocument xml = new XmlDocument();
                xml.LoadXml(_config.Log4NetConfig);
                log4net.Config.XmlConfigurator.Configure(xml.DocumentElement);
            }

            this.Schedule = ScheduleConfig.Load(_config.Schedule);

            _log.Debug(Properties.Resources.ConfigurationLoaded);

        }
        #endregion loadConfiguration

        #endregion Helper Functions
    }
}
